<?php

/*

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 $Id: option.php,v 1.10 2003/08/06 00:00:41 vollmerk Exp $
 $Source: /data/cvsroot/maint/modules/class/option.php,v $

 The subnet class definition for doing work with subnets.

*/

class Option {
	var $id;
	var $dhcp_option;
	var $dhcp_option_name;
	var $option_type;

	var $value;
	var $type;
	var $global;
	var $type_id;

	// we can populate this stuff from the database -- no sense in
	//   passing it in.
	function Option ( $option_id=0, $type=0, $global=0 ) {
		global $conf;

		$this->id   = $option_id;
		$this->type = $type;
		$this->global = $global;

		if ($this->id) { 
			$info = $this->get_info();
			$this->dhcp_option      = $info->dhcp_option;
			$this->dhcp_option_name = $this->get_option_name($this->dhcp_option);
			$this->value            = $info->value;
			$this->option_type	= $info->type;

			if ( $this->type == 'subnet' ) {
				$this->type_id = $info->subnet;
			}
			elseif ( $this->type == 'workgroup' ) {
				$this->type_id = $info->workgroup;
			}
			elseif ( $this->type == 'host' ) {
				$this->type_id = $info->host;
			}

		} // if this->id
		
		else { 
			return false;
		}


	} // Option

	function get_info () {
		global $conf;
		
		if ($this->global) { 
			$sql = "SELECT * FROM " . $this->type . "_option_global WHERE id='$this->id'";
		}

		elseif ($this->type == 'host') { 
			$sql = "SELECT * FROM host_options WHERE id='$this->id'";
		}
	
		else { 
			$sql = "SELECT * FROM $this->type"."_option_local WHERE id='$this->id'";
		}

		$db_result = mysql_query($sql, $conf['dbh']);		

		return mysql_fetch_object($db_result);
	} // get_info

	// return a VLAN object for this subnet
	function get_option_name ($option) {
		global $conf;

		$sql = "SELECT * FROM dhcp_options WHERE id=$option";
		$db_result = mysql_query($sql, $conf['dbh']);
		$obj = mysql_fetch_object($db_result);

		return $obj->name;
	} // get_option_name

	/*!
		@function check_option_value
		@discussion this is a large function that based on the dhcp option picked 
			does checks to make sure they entered a valid value. So you can't enter 
			a IP address for the lease-time and a invalid IP for the routers
	*/
	function check_option_value($dhcp_option_id, $value) { 
		global $conf;

		$dhcp_option = $this->get_option_name($dhcp_option_id);

		$ip = new IP(0);

		if (!$value) {
			$conf['error']->add_error('option_value',"Please specificy a value");
		}

		switch ($dhcp_option) { 
			case 'always-reply-rfc1048':
			case 'netbios-node-type':
			case 'time-offset':
			case 'default-ip-ttl':
			case 'default-tcp-ttl':
			case 'min-lease-time':
			case 'max-lease-time':
			case 'default-lease-time':
				if (!is_numeric($value)) { 
					$conf['error']->add_error('option_value',"Invalid Value $dhcp_option must only contain 0-9");
				}
				break;
			case 'domain-name-servers':
			case 'broadcast-address':
			case 'routers':
                                $data = explode(",", $value);
                                foreach ($data as $ip_addr) {
					if (preg_match("/\%[hi]{1}/",trim($ip_addr))) { $valid = true; }
                                        if (!$ip->is_dotted_quad(trim($ip_addr)) && !$valid) {
                                                $conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format must be 10.0.0.0,10.0.0.1");
                                        }
					unset($valid);
                                }
                                break;
			case 'ntp-servers':
			case 'swap-server':
			case 'nisplus-servers':
			case 'nntp-server':
			case 'pop-server':
			case 'resource-location-servers':
			case 'root-path':
			case 'smtp-server':
			case 'nds-servers':
			case 'cookie-servers':
			case 'netbios-name-servers':
			case 'netbios-dd-server':
			case 'www-server':
			case 'x-display-manager':
			case 'time-servers':
			case 'finger-server':
			case 'font-servers':
			case 'nis-servers':
			case 'impress-servers':
			case 'irc-server':
			case 'lpr-servers':
			case 'log-servers':
				$data = explode(",", $value);
				foreach ($data as $ip_addr) { 
					if (preg_match("/\%[hi]{1}/",trim($ip_addr))) { $valid = true; }
					if (!$ip->is_dotted_quad(trim($ip_addr)) && !$conf['ui']->is_domain($value) && !$valid) { 
						$conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format must be 10.0.0.0,10.0.0.1 or my.domain,my.domain");
					}
					unset($valid);
				}
				break;
			case 'subnet-mask':
                                $data = explode(",", $value);
				$subnet_values = array ('255','254','252','248','240','224','192','128','0');
                                foreach ($data as $ip_addr) {
					$quads = explode(".",$ip_addr);
					foreach($quads as $quad_value) {
						if (!in_array($quad_value,$subnet_values)) {
							$conf['error']->add_error('option_value',"Invalid Netmask valid values are 255,254,252,248,240,224,192,128,0");
						}
					} // end foreach quads

                                        if (!$ip->is_dotted_quad(trim($ip_addr)) ) {
                                                $conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format must be 255.255.255.0");
                                        }
                                }
                                break;
			case 'domain-name':
				if (preg_match("/\%[h]{1}/",trim($ip_addr))) { $valid = true; }
				if (!$conf['ui']->is_domain($value) && !$valid) {
					$conf['error']->add_error('option_value',"Invalid Domain.");
				}
				break;
			case 'nds-context':
			case 'nds-tree-name':
			case 'nisplus-domain':
				if (!preg_match("/^[\w\d\s\.\-]+$/",$value) ) {
					$conf['error']->add_error('option_value',"Invalid String");
				}
				break;
			case 'static-routes':
                                $data = explode(",", $value);
                                foreach ($data as $ip_route) {
					if (preg_match("/^\%[him]{1}$/",trim($ip_addr))) { $valid = true; }
                                        if (!preg_match("/^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}\s\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1.3}$/", $ip_route) && !$valid) {
                                                $conf['error']->add_error('option_value', "Invalid Value, $dhcp_option format should be 10.0.0.1 10.0.1.1, 10.0.0.20 10.0.0.23");
                                        }
					unset($valid);
                                }
                                break;

		} // end switch on dhcp_option

	} // check_option_value


	/*!
		@function update_global_option
		@discussion updates a global dhcp option
	*/
	function update_global_option($option_id,$value,$type) {
		global $conf;

		$sql = "UPDATE $type"."_option_global SET value='$value' WHERE id='$option_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

	} // update_global_option

	/*!
		@function update_host_option
		@discussion updates a host specific dhcp option
	*/
	function update_host_option($option_id, $value) {
		global $conf;

		$sql = "UPDATE host_options SET value='$value' WHERE id='$option_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

	} // update_host_option

	/*!
		@function create_global_option
		@discussion creates a global dhcp option
	*/
	function create_global_option($dhcp_option,$value,$type) {
		global $conf;

		$sql = "INSERT INTO $type"."_option_global (dhcp_option,value) VALUES ('$dhcp_option','$value')";
		$db_results = mysql_query($sql, $conf['dbh']);
		$option_id = mysql_insert_id($conf['dbh']);

	return $option_id;

	} // create_global_option

	/*!
		@function create_host_option
		@discussion creates a host specific option
	*/
	function create_host_option($dhcp_option,$value,$host_id) {
		global $conf;

		$sql = "INSERT INTO host_options (host,dhcp_option,value) VALUES ('$host_id','$dhcp_option','$value')";
		$db_results = mysql_query($sql, $conf['dbh']);
		$option_id = mysql_insert_id($conf['dbh']);

	return $option_id;

	} // create_host_option

	function delete ($type) {
		global $conf;

		$sql = "DELETE FROM $type"."_option_local WHERE id=$this->id";
		$db_result = mysql_query($sql, $conf['dbh']);
	} //delete

	/*!
		@function delete_global_option
		@discussion deletes a global option
	*/
	function delete_global_option($option_id,$type) {
		global $conf;

		$sql = "DELETE FROM $type"."_option_global WHERE id='$option_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

	} // delete_global_option

	/*!
		@function delete_host_option
		@discussion deletes a host specific dhcp option
	*/
	function delete_host_option($option_id) { 
		global $conf;

		$sql = "DELETE FROM host_options WHERE id='$option_id'";
		$db_results = mysql_query($sql, $conf['dbh']);

	} // delete_host_option

} // class Option

?>
